#The nwest function provided is a comprehensive implementation 
#of the Newey-West estimator, which is used to compute robust standard errors 
#for regression coefficients in the presence of heteroscedasticity and autocorrelation 
#up to a given lag.

#purpose: computing Newey-West adjusted heteroscedastic-autocorrelation consistent (HAC)
nwest = function(y,x,nlag){
  nobs = nrow(x);
  nvar = ncol(x);
  results = list()
  results$meth = 'nwest'
  results$y       = y;
  results$nobs    = nobs;
  results$nvar    = nvar;

#Basic regression calculation
  xpxi = solve(t(x)%*%x);
  results$beta    = xpxi%*%(t(x)%*%y);#OLS estimation
  results$yhat    = x%*%results$beta;#Predicted values
  results$resid   = y - results$yhat;#residuals
  sigu = t(results$resid)%*%results$resid;#sum of squared residuals
  results$sige    = sigu/(nobs-nvar);#variance of residuals
  
  #perform Newey-West correction
  emat = NULL;
#emat is the error matrix
  for (i in 1:nvar){
  emat = c(emat,  t(results$resid));
  }
#hhat calculates products of residuals and regressors considering lags up to 'nlag'
  hhat=emat*t(x);
  G=matrix(0, nrow = nvar, ncol = nvar); 
  w=matrix(0, nrow = (2*nlag+1), ncol = 1);
  a=0;
  while (a != (nlag+1)){
    ga=matrix(0, nrow = nvar, ncol = nvar)
    w[(nlag+1+a),1]=(nlag+1-a)/(nlag+1)
    za=hhat[,(a+1):nobs]%*%matrix(hhat[,1:(nobs-a)], ncol = nrow(hhat));
    if (a==0){
      ga=ga+za;
    }else{
    ga=ga+za+t(za);
    }
#computte the weighted sum of autocovariances
    G=G+w[(nlag+1+a),1]*ga;
    a=a+1;
  }#     end; % end of while
  
  V=xpxi*G*xpxi
#covariance matrix of regressors
  nwerr= sqrt(diag(V));
#nwerr computes robust standard errors from the diagonal of V
  results$tstat = results$beta/nwerr
#Newey-West t-statistics
  ym = y - matrix(1, nrow = nobs, ncol = 1)*mean(y);
  rsqr1 = sigu;
  rsqr2 = t(ym)%*%ym;
  results$rsqr = 1.0 - rsqr1/rsqr2# r-squared
  rsqr1 = rsqr1/(nobs-nvar);
  rsqr2 = rsqr2/(nobs-1.0);
#rsqr provide the r-squared
  results$rbar = 1 - (rsqr1/rsqr2)#
#adjusted r-squared
  ediff = results$resid[2:nobs] - results$resid[1:nobs-1];
  results$dw = t(diag((t(ediff)%*%ediff)/(sigu)))
#durbin-watson to assess autocorrelation in residuals
return(results)
  }


#designed to handle time-series regressions where the independent variables can exhibit
#multicollinearity, heteroscedasticity and autocorrelation
nvar = ncol(x);
  results = list()
  results$meth = 'nwest'
  results$y       = y;
  results$nobs    = nobs;
  results$nvar    = nvar;

#Basic regression calculation
  xpxi = solve(t(x)%*%x);
  results$beta    = xpxi%*%(t(x)%*%y);#OLS estimation
  results$yhat    = x%*%results$beta;#Predicted values
  results$resid   = y - results$yhat;#residuals
  sigu = t(results$resid)%*%results$resid;#sum of squared residuals
  results$sige    = sigu/(nobs-nvar);#variance of residuals
  
  #perform Newey-West correction
  emat = NULL;
#emat is the error matrix
  for (i in 1:nvar){
  emat = c(emat,  t(results$resid));
  }
#hhat calculates products of residuals and regressors considering lags up to 'nlag'
  hhat=emat*t(x);
  G=matrix(0, nrow = nvar, ncol = nvar); 
  w=matrix(0, nrow = (2*nlag+1), ncol = 1);
  a=0;
  while (a != (nlag+1)){
    ga=matrix(0, nrow = nvar, ncol = nvar)
    w[(nlag+1+a),1]=(nlag+1-a)/(nlag+1)
    za=hhat[,(a+1):nobs]%*%matrix(hhat[,1:(nobs-a)], ncol = nrow(hhat));
    if (a==0){
      ga=ga+za;
    }else{
    ga=ga+za+t(za);
    }
#computte the weighted sum of autocovariances
    G=G+w[(nlag+1+a),1]*ga;
    a=a+1;
  }#     end; % end of while
  
  V=xpxi*G*xpxi
#covariance matrix of regressors
  nwerr= sqrt(diag(V));
#nwerr computes robust standard errors from the diagonal of V
  results$tstat = results$beta/nwerr
#Newey-West t-statistics
  ym = y - matrix(1, nrow = nobs, ncol = 1)*mean(y);
  rsqr1 = sigu;
  rsqr2 = t(ym)%*%ym;
  results$rsqr = 1.0 - rsqr1/rsqr2# r-squared
  rsqr1 = rsqr1/(nobs-nvar);
  rsqr2 = rsqr2/(nobs-1.0);
#rsqr provide the r-squared
  